clear;clc;
warning off all
load electricity.mat;
load comb_electricity_spot.mat;
X_seas=log(COMBelectricityprices)';
X=log(electricity(1:end-1,1))';
Y=log(electricity(1:end-1,2:end));
T=electricity(end,2:end);
delta=1/252;
N=length(X);
Precision=1000000;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-30,...
                     'UseParallel','always',...
                     'TolFun',1e-30);

Step=14;
x=X_seas(1:Step:end);
x0=[3.690,5.72,5.059,6.9905;];
[vecSeas, fval1]=fminsearch(@(vec) minimizeSeasonality(vec,x,delta*Step),x0,options);
vecSeas(1)=0;
disp(vecSeas);
disp(fval1);
%%
% vecSeas(1)=0;
% t=(0:length(X_seas)-1)*delta;
% Yseas=X_seas-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t);
%  figure(1)
% plot(t,seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t),t,X_seas')
%  figure(2)
% plot(t/delta,Yseas)


%%
samplesize=200;
t=(534:534+samplesize-1)*delta;
%%%%%%%%%%%%%%%%%%%DESEASONALISED PRICES%%%%%%%%%%%%%%%%%%%
Yseas=X(1:samplesize)-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t);
%%  
%%  
seasonalityParameters=vecSeas;
initcond=[      1,      10,     100,    2,    5     ;...
          .09, 1, 2, 2, -.3;...
            .0001   .0001   .001    -2    .0001 ] ;
delta1=samplesize*delta;
%%%%%%%%%%%%%%%%MOMENT_MATCHING USE 'fminsearch'%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%Lambda=1
nMoments=9;
Lambda=ones(1,nMoments);
M=moments_of_data(Yseas,nMoments);
[theta1,fval1]=fminsearch(@(vec) minimizeMomentsJF(...
    M,@momentsofOU2Fv2,vec,Lambda,Yseas(1),delta1), initcond(2,:),options);
disp(fval1);
%theta1(7)=sin(theta1(7));
disp(theta1);
%%%%%%%%%%%%%%%Calibrate covariance%%%%%%
[Lambda,fvalLambda]=fminsearch(@(Lambda) minimizeMomentsJF(...
    M,@momentsofOU2Fv2,theta1,Lambda,Yseas(1),delta1), Lambda,options);
disp(Lambda);
disp(fvalLambda);


%%%%%%%%%%%%%%%Calibrate Jumps %%%%%%%%%%%
initval=[      1,      10;...
                10, 0.00001;...
            .0001   .0001 ] ;

%%%%%%%%%%%%%%%%MOMENT_MATCHING USE 'fminsearch'%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%Lambda=1

[jump,fval1]=fminsearch(@(jump) minimizeMoments(...
    M,@momentsofOU2Fv2,theta1,jump,Lambda,Yseas(1),delta1), initval(2,:),options);
disp(fval1);
%theta1(7)=sin(theta1(7));

%%%%%%%%%%%%%%%Calibrate covariance%%%%%%
% [Lambda,fvalLambda]=fminsearch(@(Lambda) minimizeMoments(...
%     M,@momentsofOU2Fv2,theta1,jump,Lambda,Yseas(1),delta1), Lambda,options);
theta1=[theta1, jump];
disp(theta1);
disp(Lambda);
disp(fvalLambda);
theta1(6)=abs(theta1(6))+10;
                %% Calibrate Risk
Precision=100;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-20,...
                     'UseParallel','always',...
                     'TolFun',1e-20);
nVec=ones(1,length(T))*.99;
rVec0=[0,0,0];
[rVec, fval2]=fminsearch(@(rVec) MLEndim(nVec,rVec,Y(1:100,:),Yseas,...
                    @OUfutprice2F,delta,0,T,theta1,vecSeas),rVec0,options);
disp(fval2);
disp(rVec);     


            %% Calibrate Noise Matrix
Precision=100;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-20,...
                     'UseParallel','always',...
                     'TolFun',1e-20);
C0=[1 1 1 1 1 1 1];
[nVec, fval3]=fminsearch(@(nVec) MLEndim(nVec,rVec,Y(1:100,:),Yseas,...
                    @OUfutprice2F,delta,0,T,theta1,vecSeas),C0,options);
C=diag(abs(nVec)+0.01);

disp(fval3);
disp(C);             
%% Set Up Particle filter
nParts=100;
particles=[Yseas(1)+randn([nParts 1])*sqrt(M(2)/samplesize),zeros(nParts,1); zeros(nParts*N,2)];
%W=[ones(nParts,1)*1/nParts,zeros(nParts,N-1)];
W=[normpdf(particles(1:nParts,1),Yseas(1),sqrt(M(2)/samplesize))./sum(normpdf(particles(1:nParts,1),Yseas(1),sqrt(M(2)/samplesize))),zeros(nParts,N-1)];
problem2.generator=@OUgenerator2F;
problem2.futprice=@OUfutprice2F;
X_filtered=zeros(N-2,2);
tf=(2:N)*delta;
RMSE=zeros(length(T),N-1);
MRAEi=zeros(length(T),N-1);
%% PARTICLE FILTER LOOP

for i=1:N-1
    X_filtered(i,:)=W(:,i)'*particles((i-1)*nParts+1:i*nParts,:);
    [particles(i*nParts+1:(i+1)*nParts,:), W(:,i+1), RMSE(:,i), MRAEi(:,i)]=...
        particle_filter(problem2,theta1,vecSeas,Y(i+1,:)',C,delta,tf(i)...
                ,T,W(:,i),X_filtered(i,:),rVec);
    
end;

%% %%%%%%%%%%%%%%%Plot DATA%%%%%%%%%%%
tp=tf+534*delta;
a=plot(real(X_filtered(:,1)),'k');hold on
xlabel('Day Of reading');
ylabel('Log-Price')
b=plot(real(X_filtered(:,2)),'y');
A=real(X_filtered(:,2))+real(X_filtered(:,1));%*exp(-abs(theta1(3))*delta);
c=plot(A,'--');
hold on
d=plot(X(2:N)-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),tp),'r');
legend([a,b,c,d],'Process x_t', 'Process \zeta_t','x_t+\zeta_t' ,'Observed price, deseasonalised','Location','Best');
%%
figure(2)
    mesh(tp/delta,T*delta,RMSE);
    ylabel('Contract Maturity');
    xlabel('Observed date');
%%
figure(3)
    MRAE=mean(MRAEi,2);
    plot(T*delta,MRAE*100,'.-');
    xlabel('Contract Maturity');
    ylabel('MRAE \%');