clear;clc;
warning off all
load electricity.mat;
%% %%INITIAL DATA STRUCTURE%%%%%%%%%%%%%%
startpoint=1;
N=600;
insamplesize=N/2;
delta=1/252;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X=electricity(startpoint:startpoint+N-1,1)';
Y=log(electricity(startpoint:startpoint+N-1,2:end));
T=[electricity(end,2:end)];

Precision=1000;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-30,...
                     'UseParallel','always',...
                     'TolFun',1e-30);

Step=14;
x=X(1:Step:end);
x0=[3.690,5.72,5.059,6.9905;];
[vecSeas, fval1]=fminsearch(@(vec) minimizeSeasonality(vec,x,delta*Step),x0,options);
vecSeas(1)=min(X-.1);
disp(vecSeas);
disp(fval1);

%%
t=(startpoint-1:startpoint+N-2)*delta;
Yseas=log(X-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t));
for i=1:length(T)
    YseasF(:,i)=log(exp(Y(:,i))'-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t));
end;
%%
%  figure(1)
% plot(t,seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t),t,X')
%  figure(2)
% plot(t/delta,log(Yseas),t/delta,YseasF)

vecSeas=[0 0 0 0];
%%  
seasonalityParameters=vecSeas;
initcond=[      1,      10,     100 123;...
            .8, .1, .5, Yseas(1);...
            .0001   .0001   .001 ,0] ;


            %% Calibrate Risk and Noise Matrix
Precision=10000;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-32,...
                     'UseParallel','always',...
                     'TolFun',1e-32);
rVec0=[-1,1];
C0=[rVec0 initcond(2,:) ones(1,length(T))*.99;];
[nVec, fval3]=fminsearch(@(nVec) MLEndim(nVec,Y(1:insamplesize,:),Yseas,...
                    @futprice,delta,t(1),T,vecSeas),C0,options);
                
C=diag(abs(nVec(7:end))+0.01);
rVec=nVec(1:2);
theta1=nVec(3:6);
disp(fval3);
disp(C);             
disp(theta1);
%% Set Up Particle filter
%N=N-60;
nParts=100;
particles=[Yseas(1)+randn([nParts 1]),zeros(nParts,1); ones(nParts*N,2)*theta1(1)];
%W=[ones(nParts,1)*1/nParts,zeros(nParts,N-1)];
W=[ones(nParts,N-1)/nParts,zeros(nParts,N-1)];
problem2.generator=@generatorR;
problem2.futprice=@futprice;
X_filtered=zeros(N-2,2);
tf=((startpoint:startpoint+N-1))*delta;
RMSE=zeros(length(T),N-1);
MRAEi=zeros(length(T),N-1);

%% PARTICLE FILTER LOOP

for i=1:N-1
    X_filtered(i,:)=W(:,i)'*particles((i-1)*nParts+1:i*nParts,:);
    [particles(i*nParts+1:(i+1)*nParts,:), W(:,i+1), RMSE(:,i), MRAEi(:,i)]=...
        particle_filter(problem2,theta1,vecSeas,Y(i+1,:)',C,delta,tf(i)...
                ,T,W(:,i),X_filtered(i,:),rVec);
    
end;

%% %%%%%%%%%%%%%%%Plot DATA%%%%%%%%%%%
figure(1);clf;
tp=tf;
hold on
xlabel('Day Of reading');
ylabel('Log-Price')
A=real(real(X_filtered(:,1)*exp(-abs(theta1(3))*delta)));
c=plot(A,':');
hold on
d=plot(Yseas(2:N),'r');
legend([c,d],'Process x_t', 'Observed price, deseasonalised','Location','Best');
%%
figure(2)
    RMSE=mean(RMSE,2);
    plot(T*delta,RMSE);
    ylabel('RMSE');
    xlabel('MATURITY');
%%
figure(3)
    MRAE=mean(MRAEi(:,1:N-100),2);
    
    plot(T*delta,MRAE*100,'.-');
    xlabel('Contract Maturity');
    ylabel('MRAE \%');