clc;
clear;
load gasoline.mat;
load HBnatgas.mat;
load lightoil.mat;
warning off all;
global  calType;
calType='0';
matlabpool open 4;
Precision=20000; %number of function evaluations and maximum iterations
calibrationsample=300; % number of datapoints for calibration
testsample=301; % number of datapoints for out-of-sample performance
tic;
%% Kalman filter Approach for Basic vasicek one-state model 
k=1; % model number
initval=[.1 1 .2 2 1000]; % alpha, kappa, sigma, c, P0
n=length(initval);
Nparam(k)=n;
func_handle=@onestatefilter;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);

%% Kalman filter Approach for Vasicek one-state model with price of risk 
k=2;
initval=[.1 1 .2 2 .1 10]; % alpha, kappa, sigma, c, lambda,P0
n=length(initval);
Nparam(k)=n;
func_handle=@onestatefilter_rf;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);


%% Kalman filter Approach for Vasicek one-state model with price of risk and seasonality;
k=3;
initval=[.1 1 .2 2 .1 .2 .3 .4 1000]; % alpha, kappa, sigma, c, lambda, phi, zeta, psi, P0
n=length(initval);
Nparam(k)=n;
func_handle=@seasonalityfilter;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);

%% Kalman filter Approach for Vasicek one-state model with stochastic alpha and seasonality;
k=4;
initval=[.1 1 .2 2 .1 .2 .3 .4 .2 10]; % alpha, kappa, sigma, c, lambda, phi, zeta, psi, eta, P0
n=length(initval);
Nparam(k)=n;
func_handle=@stochalphafilter;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);

%% Kalman filter Approach for two-state model
k=5;
initval=[.1 1 .2 .2 .2 .2 .1 2 2 10 .1 ]; % alpha, kappa1,kappa2, sigma1,sigma2, 
                                              %c, beta,lambda1,lambda2,
                                              %P01, P02
n=length(initval);
Nparam(k)=n;
func_handle=@twostatefilter;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);

%% Kalman filter Approach for Vasicek two-state model with seasonality;
k=6;
initval=[.1 1 .2 .2 .2 .2 .1 2 2 10 .1 .2 .3 .4 ]; % alpha, kappa1,kappa2, sigma1,sigma2, 
                                              %c, beta,lambda1,lambda2,
                                              %P01, P02, phi,zeta,psi
n=length(initval);
Nparam(k)=n;
func_handle=@twostatefilterseasonality;
display(['Evaluating ', num2str(k), ' function', func2str(func_handle)]);
[gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),gasolineRMSE(k),gasolinefval(k)]=main(func_handle,gasoline,initval,Precision,calibrationsample,testsample);
[natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),natgasfval(k)]=main(func_handle,HBnatgas,initval,Precision,calibrationsample,testsample);
[lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),lightoilRMSE(k),lightoilfval(k)]=main(func_handle,lightoil,initval,Precision,calibrationsample,testsample);

toc;


%% AIC Test
%AICc values
AICgasoline=gasolinefval+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
AICnatgas=natgasfval+2*Nparam+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
AIClightoil=lightoilfval+2*Nparam+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
%delta AIC
dAICgasoline=AICgasoline-min(AICgasoline);
dAICnatgas=AICnatgas-min(AICnatgas);
dAIClightoil=AIClightoil-min(AIClightoil);
%AIC weights
wAICgasoline=exp(-dAICgasoline/2)./(sum(exp(-dAICgasoline/2)));
wAICnatgas=exp(-dAICnatgas/2)./(sum(exp(-dAICnatgas/2)));
wAIClightoil=exp(-dAIClightoil/2)./(sum(exp(-dAIClightoil/2)));
%% Save data to excel
excelwrite;
matlabpool close;


%% Saving Thetas

xlswrite(filename,...
    [Nparam',gasolineTheta],'Sheet2','B3:p8');

xlswrite(filename,...
    [Nparam',lightoilTheta],'Sheet2','B11:p16');

xlswrite(filename,...
    [Nparam',natgasTheta],'Sheet2','B20:p25');