clear;clc;
warning off all
load electricity.mat;
if matlabpool('size')==0
    matlabpool open local 4;
end
%% %%INITIAL DATA STRUCTURE%%%%%%%%%%%%%%
startpoint=301;
N=300;
insamplesize=N/2;
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X=log(electricity(startpoint:startpoint+N-1,1))';
Y=log(electricity(startpoint:startpoint+N-1,2:end));
T=electricity(end,2:end);
delta=1/252;
N=length(X);
Precision=1000000;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'TolX',1e-30,...
                     'UseParallel','always',...
                     'TolFun',1e-30);

Step=14;
x=X(1:Step:end);
x0=[3.690,5.72,5.059,6.9905;];
[vecSeas, fval1]=fminsearch(@(vec) minimizeSeasonality(vec,x,delta*Step),x0,options);
%vecSeas(1)=0;
disp(vecSeas);
disp(fval1);
%%
% vecSeas(1)=0;
% t=(0:length(X_seas)-1)*delta;
% Yseas=X_seas-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t);
%  figure(1)
% plot(t,seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t),t,X_seas')
%  figure(2)
% plot(t/delta,Yseas)


%%
t=(startpoint-1:startpoint+insamplesize-2)*delta;
%%%%%%%%%%%%%%%%%%%DESEASONALISED PRICES%%%%%%%%%%%%%%%%%%%
Yseas=X(1:insamplesize)-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),t);
%%  
Precision=10000;
options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'Algorithm','interior-point',...
                     'Display','off',...
                     'TolX',1e-30,...
                     'UseParallel','always',...
                     'TolFun',1e-30);

seasonalityParameters=vecSeas;
initcond=[  5    5,     1,     10,      10,    10           1   100      2;...
                  0.2,   0,      .2,   .4,       .2,     .2            0.1    2      .2;...
                -5  -1   .0001   .001   .0001    .001  -1 0.01    0.01] ;



delta1=insamplesize*delta;
%% %%%%%%%%%%%%%%MOMENT_MATCHING USE 'fmincon'%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%Lambda=1
nMoments=9;
nVar=length(initcond(2,:));
A=zeros(nVar);
A(1,[1 2])=1;
b=[Yseas(1);zeros(nVar-1,1)];
Lambda=ones(1,nMoments);
M=moments_of_data(Yseas,nMoments);
[theta1,fval1]=fmincon(@(vec) minimizeMoments(...
    M,@momentsofOU2Fv2,vec,Lambda,Yseas(1),delta1,nMoments), initcond(2,:),[],[]...
    ,A,b,initcond(3,:),initcond(1,:),[],options);
disp(fval1);
%theta1(7)=sin(theta1(7));
disp(theta1);
%% %%%%%%%%%%%%%Calibrate covariance%%%%%%
[Lambda,fvalLambda]=fminsearch(@(Lambda) minimizeMoments(...
    M,@momentsofOU2Fv2,theta1,Lambda,Yseas(1),delta1,nMoments), Lambda,options);
disp(Lambda);
disp(fvalLambda);
%% Recalibrate
[theta1,fval1]=fmincon(@(vec) minimizeMoments(...
    M,@momentsofOU2Fv2,vec,Lambda,Yseas(1),delta1,nMoments), theta1,[],[]...
    ,A,b,initcond(3,:),initcond(1,:),[],options);
disp(fval1);

disp(theta1);

%% PARTICLE FILTER LOOP%% Set Up Particle filter
nParts=2^10;
particles=[theta1(1)*ones(nParts,1),theta1(2)*ones(nParts,1); zeros(nParts*N,2)];
%W=[ones(nParts,1)*1/nParts,zeros(nParts,N-1)];
W=[1/nParts*ones(nParts,1),zeros(nParts,N-1)];
problem2.generator=@OUgenerator2F;
problem2.futprice=@OUfutprice2F;
X_filtered=[theta1(1) theta1(2); zeros(N-1,2)];
tf=(startpoint:startpoint+N-2)*delta;
RMSEi=zeros(length(T),N-1);
MRAEi=zeros(length(T),N-1);
Noise=cell(N,1);
Noise{1}=eye(length(T));
risk=zeros(N,3);
%% PARTICLE FILTER LOOP

for i=1:N-1
    [risk(i+1,:), Noise{i+1},X_filtered(i+1,:), particles(i*nParts+1:(i+1)*nParts,:), W(:,i+1), RMSEi(:,i), MRAEi(:,i)]=...
        particle_filter(problem2,theta1,vecSeas,Y(i+1,:)',Noise{i},delta,tf(i)...
                ,T,W(:,i),X_filtered(i,:),risk(i,:),i);
    
end;

%% %%%%%%%%%%%%%%%Plot DATA%%%%%%%%%%%
f1=figure(1);clf;
tp=tf;
a=plot(real(X_filtered(:,1)),'.');hold on
xlabel('Day Of reading');
ylabel('Log-Price')
b=plot(real(X_filtered(:,2)),'.-');
A=real(X_filtered(:,2))+real(X_filtered(:,1)*exp(-abs(theta1(3))*delta))+...
    A_term(theta1,risk(N,:),delta)+integral_jump([abs([theta1(8),theta1(9)]),risk(end,3),abs(theta1(3))],delta,0);%*exp(-abs(theta1(3))*delta);
c=plot(A,':');
hold on
d=plot(X(2:N)-seasonality(vecSeas(1),vecSeas(2),vecSeas(3),vecSeas(4),tp),'r');
legend([a,b,c,d],'Process x_t', 'Process \zeta_t','x_t+\zeta_t' ,'Observed price, deseasonalised','Location','Best');
print(f1,'-dpsc','Filtered_process_w_Jump_ex2.eps')
%%
f2=figure(2);clf
    RMSE=sqrt(mean(RMSEi,2));
    plot(T*delta,RMSE);
    ylabel('RMSE');
    xlabel('MATURITY');
print(f2,'-dpsc','Prediction_RMSE_w_Jump_ex2.eps')
%%
f3=figure(3);clf;
    MRAE=mean(MRAEi,2);
    
    plot(T*delta,MRAE*100,'.-');
    xlabel('Contract Maturity');
    ylabel('MRAE \%');
print(f3,'-dpsc','Prediction_MRAE_w_Jump_ex2.eps')
%%
f4=figure(4);clf;
NOISE=cell2mat(Noise); hold on
color={'dr','xb','xg','om','+k','xc'};
x=1:300;
leg=cell(6,1);
names=cell(6,1);
for i=1:length(T)
    leg{i}=plot(x,NOISE(i:6:end,i),color{i}); 
    names{i}=strcat(num2str(    T(i)/delta), ' days');
end
xlabel('Day');
ylabel('Noise Level');
legend(cell2mat(leg),names)
%%
f5=figure(5);clf;
hold on
color={'xr','-b','.g'};
x=1:300;
leg=cell(3,1);
names={'h_x','h_\zeta','\beta'};
for i=1:3
    leg{i}=plot(x,risk(:,i),color{i}); 
end

xlabel('Day');
ylabel('Risk Level');
legend(cell2mat(leg),names)
%%
if matlabpool('size')~=0
    matlabpool close;
end;