clc;
clear;
load gasoline.mat;
load HBnatgas.mat;
load lightoil.mat;
warning off all;
%Choice of calibration algorithm GA for genetic algorithm, any other value
%for fminsearch
global  calType;
calType='GA1';
%Parallelise the problem: set number of workers to 4;
%matlabpool open local 4;
Precision=20000; %number of function evaluations and maximum iterations 
%in case of genetic algorithm number of generations.
calibrationsample=300; % number of datapoints for calibration
testsample=301; % number of datapoints for out-of-sample performance
%% Setup parameters for parallel computation

%% Setup parameters for parallel computation

% funchandle is a vector of functions we are going to test
%                                        k
func_handle={@onestatefilter,...        %1
            @onestatefilter_rf,...      %2
            @seasonalityfilter,...      %3   
            @twostatefilter,...         %4
            @twostatefilterseasonality};%5
        
% initval_M is a vector of initial values for calibration with following
% structure:
%[Number of parameters initial_values]


initval_M=[5   10 10 1 1 1000 0 0 0 0 0 0 0 0 0;... % alpha, kappa, sigma, c, P0
           6   .1 1 .2 2 .1 0 0 0 0 0 0 0 0 0;...     % alpha, kappa, sigma, c, P0, lambda
           10    .1 1 .2 2 .1 0 0 0 0  0 0 0 0 0;...    %alpha, kappa, sigma, c, P0, lambda, phi, zeta, psi
           10    .1 1 .2 .2 .2 .2 .1 2 2 10 .1 0 0 0;...     % alpha, kappa1,kappa2, sigma1,sigma2 c, beta, lambda1, lambda2,  P01, P02
           14    .1 1 .2 .2 .2 .2 .1 2 2 10 .1 0 0 0 ];     % alpha, kappa1,kappa2, sigma1,sigma2 c, beta, lambda1, lambda2,  P01, P02, phi, zeta, psi

%% Initialise containers for outputs
            gasolineTheta=zeros(5,12);
            natgasTheta=zeros(5,12);
            lightoilTheta=zeros(5,12);
            gasolineRAE=zeros(calibrationsample,6);
            gasolineRMSE=zeros(1,5);
            gasolinefval=zeros(1,5);
            gasolineMRAE=zeros(1,5);
            natgasRAE=zeros(calibrationsample,6);
            natgasRMSE=zeros(1,5);
            natgasfval=zeros(1,5);
            natgasMRAE=zeros(1,5);
            lightoilRAE=zeros(calibrationsample,5);
            lightoilRMSE=zeros(1,5);
            lightoilfval=zeros(1,5);
            lightoilMRAE=zeros(1,5);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%            
tic;
%% Calibration Gasoline
for k=1:5
n=round(initval_M(k,1));
        display(['Evaluating ', num2str(k), ' function ', func2str(func_handle{k})]);
        [gasolineTheta(k,1:n),gasolineRAE(:,k),gasolineMRAE(k),...
            gasolineRMSE(k),gasolinefval(k)]=main(func_handle{k},...
            gasoline,initval_M(k,2:(1+initval_M(k,1))),...
            Precision,calibrationsample,testsample);
        if k<6
            initval_M(k+1,2:(1+initval_M(k,1)))=gasolineTheta(k,1:n);
        end;
end;             
gasolineTheta(:,2)=abs(gasolineTheta(:,2))+0.01;
gasolineTheta(1:3,3:4)=abs(gasolineTheta(1:3,3:4));
gasolineTheta(1:3,4)=abs(gasolineTheta(1:3,4))+.01;
gasolineTheta(4:5,3:4)=abs(gasolineTheta(4:5,3:4));
gasolineTheta(4:5,5)=abs(gasolineTheta(4:5,5))+0.01;
gasolineTheta(1:3,5)=abs(gasolineTheta(1:3,5));
gasolineTheta(4:5,9:10)=abs(gasolineTheta(4:5,9:10))+.0001;
gasolineTheta(4:5,2)=abs(gasolineTheta(4:5,2))+0.01;
gasolineTheta(4:5,8)=sin(gasolineTheta(4:5,8));
%% Calibration Natgas
initval_M(5,2:12)=[1.26 2 2 2 2 2 2 2 2 2 2];
for k=1:5
   
        n=round(initval_M(k,1));
        display(['Evaluating ', num2str(k), ' function ', func2str(func_handle{k})]);
        [natgasTheta(k,1:n),natgasRAE(:,k),natgasMRAE(k),natgasRMSE(k),...
            natgasfval(k)]=main(func_handle{k},...
            HBnatgas,initval_M(k,2:(1+initval_M(k,1))),...
            Precision,calibrationsample,testsample);
        if k<6
         initval_M(k+1,2:(1+initval_M(k,1)))=natgasTheta(k,1:n);
        end;
     
end;
natgasTheta(:,2)=abs(natgasTheta(:,2))+0.01;
natgasTheta(1:3,3:4)=abs(natgasTheta(1:3,3:4));
natgasTheta(1:3,4)=abs(natgasTheta(1:3,4))+.01;
natgasTheta(4:5,3:4)=abs(natgasTheta(4:5,3:4));
natgasTheta(4:5,5)=abs(natgasTheta(4:5,5))+0.01;
natgasTheta(1:3,5)=abs(natgasTheta(1:3,5));
natgasTheta(4:5,9:10)=abs(natgasTheta(4:5,9:10))+.0001;
natgasTheta(4:5,2)=abs(natgasTheta(4:5,2))+0.01;
natgasTheta(4:5,8)=sin(natgasTheta(4:5,8));
%% Calibration Ligh Oil
initval_M(1,2:6)=[5 10 .1 1 100];
for k=1:5
        n=round(initval_M(k,1));
        display(['Evaluating ', num2str(k), ' function ', func2str(func_handle{k})]);
        [lightoilTheta(k,1:n),lightoilRAE(:,k),lightoilMRAE(k),...
            lightoilRMSE(k),lightoilfval(k)]=main(func_handle{k},...
            lightoil,initval_M(k,2:(1+initval_M(k,1))),...
            Precision,calibrationsample,testsample);
        if k<6
          initval_M(k+1,2:(1+initval_M(k,1)))=lightoilTheta(k,1:n);
        end;
       
end;            
toc;
%%
lightoilTheta(:,2)=abs(lightoilTheta(:,2))+0.01;
lightoilTheta(1:3,3:4)=abs(lightoilTheta(1:3,3:4));
lightoilTheta(1:3,4)=abs(lightoilTheta(1:3,4))+.01;
lightoilTheta(4:5,3:4)=abs(lightoilTheta(4:5,3:4));
lightoilTheta(4:5,5)=abs(lightoilTheta(4:5,5))+0.01;
lightoilTheta(1:3,5)=abs(lightoilTheta(1:3,5));
lightoilTheta(4:5,9:10)=abs(lightoilTheta(4:5,9:10))+.0001;
lightoilTheta(4:5,2)=abs(lightoilTheta(4:5,2))+0.01;
lightoilTheta(4:5,8)=sin(lightoilTheta(4:5,8));
Nparam=initval_M(1:5,1)';

%% AIC Test
%AICc values
AICgasoline=gasolinefval+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
AICnatgas=natgasfval+2*Nparam+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
AIClightoil=lightoilfval+2*Nparam+2*Nparam+(2*Nparam.*(Nparam+1))./(calibrationsample-Nparam-1);
%% delta AIC
dAICgasoline=AICgasoline-min(AICgasoline);
dAICnatgas=AICnatgas-min(AICnatgas);
dAIClightoil=AIClightoil-min(AIClightoil);
%% AIC weights
wAICgasoline=exp(-dAICgasoline/2)./(sum(exp(-dAICgasoline/2)));
wAICnatgas=exp(-dAICnatgas/2)./(sum(exp(-dAICnatgas/2)));
wAIClightoil=exp(-dAIClightoil/2)./(sum(exp(-dAIClightoil/2)));
%% Save data to excel file
if strcmp(calType,'GA')
    filename='results2.xlsx';
else
    filename='results.xlsx';
end;

%% Writing data
% Writing Gasoline Data to file
xlswrite(filename,...
    [Nparam',gasolineMRAE',gasolineRMSE',-gasolinefval'/2,...
    AICgasoline',dAICgasoline',wAICgasoline'],'C3:I8');


% Writing Natgas Data to file

xlswrite(filename,...
    [Nparam',natgasMRAE',natgasRMSE',-natgasfval'/2,...
    AICnatgas',dAICnatgas',wAICnatgas'],'C11:I15');


% Writing Lightoil Data to file


xlswrite(filename,...
    [Nparam',lightoilMRAE',lightoilRMSE',-lightoilfval'/2,...
    AIClightoil',dAIClightoil',wAIClightoil'],'C19:I23');

% Saving Thetas

xlswrite(filename,...
    [Nparam',gasolineTheta],'Sheet2','B3:P7');

xlswrite(filename,...
    [Nparam',natgasTheta],'Sheet2','B11:P15');

xlswrite(filename,...
    [Nparam',lightoilTheta],'Sheet2','B20:P24');
%%
%matlabpool close;