%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function onestatefilter_rf
% Author: Suren Islyaev
% Input:
%     c=vector of parameters
%     y=futures price data
%     x0=initial state of the filter
%     delta = timestep
%     T=time to maturity
%     tag= method used(calibration or test for one step ahead forecast)
% Output:
%     z=estimated futures price one step ahead vector or MLE depends on tag
%     value
%     commodity = estimated commodity price one step ahaed value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [z,commodity ]= onestatefilter_rf(c,y,x0,delta,T,tag,~)
[m, n]=size(y);
%%%% Following parameter transformations were implemented to avoid getting
%%%% wrong parameter valued during optimisation, because Nelder-Mead does
%%%% unconstrained optimisation
mu=c(1);                %mu
theta=abs(c(2));        %theta
kap=abs(c(3))+0.01;     %kappa
Sigma=abs(c(4));        %sigma
Q=eye(n)*(abs(c(5))+0.01);  %Noise matrix
QQ=Q^2;     
p=abs(c(6));            %state covariance
lambda=c(7);            %risk premium



%initialise vectors and Matrices
x_old=[x0 zeros(1,m)];  %x_{n|n}
x_new=zeros(1,m+1);     %x_{n|n+1}
%For more details of further values see the paper 
an=mu+lambda*Sigma;    
B=exp(-kap*delta);
g=mu/kap*(1-exp(-kap*delta));
RR=Sigma^2*0.5/kap*(1-exp(-2*kap*delta))+theta/kap/kap*(1-exp(-kap*delta))^2;
L=T'*ones(1,m+2);
j=ones(n,1)*(0:m+1).*delta;
Tc=(L-j);
A=exp(-kap*Tc);
At=A';
y=y';
BB=B^2;
d=(an)/kap*(1-exp(-kap*Tc))+Sigma^2*0.25/kap*(1-exp(-2*kap*Tc))+theta/2/kap/kap*(1-exp(-kap*Tc)).^2;
switch lower(tag)
    case {'calibration'}
        z=0;
        for i=2:m
        x_new(:,i-1)=B*x_old(:,i-1)+g;   %state estimate x_{n|n+1}
        v=y(:,i)-(A(:,i)*x_new(:,i-1)+d(:,i));  % innovation
        sig=A(:,i)*p*At(i,:)+QQ;                % innovation covariance
        k=p*At(i,:)/sig;                        % Kalman gain
        x_old(:,i)=x_new(:,i-1)+k*v;            % posteriori state estimate
        p=BB*p+RR-BB*p^2*At(i,:)*(sig\A(:,i));  % posteriori covariance
        z=z+log(det(sig))+v'*(sig\v);           % Add MLE
        end;
       
    
    case {'test'}
        commodity=B*x_old(:,1)+g;               %state estimate apriopri
        z=A(:,2)*commodity+d(:,2);              %Futures state estimate apriori
        
end

