%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function seasonalityfilter
% Author: Suren Islyaev
% Input:
%     c=vector of parameters
%     y=futures price data
%     x0=initial state of the filter
%     delta = timestep
%     T=time to maturity
%     tag= method used(calibration or test for one step ahead forecast)
%     D = day of reading
% Output:
%     z=estimated futures price one step ahead vector or MLE depends on tag
%     value
%     commodity = estimated commodity price one step ahaed value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [z, commodity] = seasonalityfilter(c,y,x0,delta,T,tag,D)
[m, n]=size(y);
%%%% Following parameter transformations were implemented to avoid getting
%%%% wrong parameter valued during optimisation, because Nelder-Mead does
%%%% unconstrained optimisation
mu=c(1);
theta=abs(c(2));
kap=abs(c(3))+0.01;
Sigma=abs(c(4));
Q=eye(n)*(abs(c(5))+0.01);
QQ=Q^2;
p=abs(c(6));
lambda=c(7);
% seasonality parameters
ph=c(9);
zeta=c(9);
psi=c(10);
b=c(11);


alpha = @(b,zeta,psi,ph,t) b+zeta*sin(psi*t+ph); % seasonality function

%initialise vectors and Matrices
x_old=[x0 zeros(1,m)];  %x_{n|n}
x_new=zeros(1,m+1);     %x_{n|n+1}
%For more details of further values see the paper 
B=exp(-kap*delta);
g=mu/kap*(1-exp(-kap*delta));
RR=Sigma^2*0.5/kap*(1-exp(-2*kap*delta))+theta/kap/kap*(1-exp(-kap*delta))^2;
L=T'*ones(1,m+2);
j=ones(n,1)*(0:m+1).*delta;
Tc=(L-j);
A=exp(-kap*Tc);
y=y';
BB=B^2;
dg=alpha(b,zeta,psi,ph,D)';
slk=Sigma*lambda; 
d=(mu+slk)./kap.*(1-exp(-kap*Tc))+Sigma^2*0.25/kap*(1-exp(-2*kap*Tc))+theta/2/kap/kap*(1-exp(-kap*Tc)).^2;

switch lower(tag)
    case {'calibration'}
        z=0;
        for i=2:m
        sig=A(:,i)*p*A(:,i)'+QQ;                % state estimate x_{n|n+1}
        k=p*A(:,i)'/sig;                        % innovation
        x_new(:,i)=B*x_old(:,i)+g;              % innovation covariance
        p=BB*p+RR-BB*p^2*A(:,i)'/sig*A(:,i);    % Kalman gain
        v=y(:,i)-(A(:,i)*x_new(:,i)+d(:,i)+dg); % posteriori state estimate
        z=z+log(det(sig))+v'/sig*v;             % posteriori covariance
        x_old(:,i+1)=x_new(:,i)+k*v;            % Add MLE
        end;
     

    case {'test'}
        commodity=B*x_old(:,1)+g;               % state estimate apriopri
        z=A(:,2)*commodity+d(:,2)+dg;           % Futures state estimate apriori

end

