%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function twostatefilterseasonality
% Author: Suren Islyaev
% Kalman filter is used for 2 stete process witseasonality
% Input:
%     c=vector of parameters
%     y=futures price data
%     x0=initial state of the filter
%     delta = timestep
%     T=time to maturity
%     tag= method used(calibration or test for one step ahead forecast)
% Output:
%     z=estimated futures price one step ahead vector or MLE depends on tag
%     value
%     commodity = estimated commodity price one step ahaed value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [z, commodity]=twostatefilterseasonality(c,y,x0,delta,T,tag,D)
[m, n]=size(y);
%%%% Following parameter transformations were implemented to avoid getting
%%%% wrong parameter valued during optimisation, because Nelder-Mead does
%%%% unconstrained optimisation
a2=c(1);            %alpha_2
k=abs(c(2))+0.01;   %kappa
sigma1=abs(c(3));   %sigma_1
sigma2=abs(c(4));   %sigma_2
Q=eye(n)*(abs(c(5))+0.01);  %Measurement erro matrix
QQ=Q^2;     
l1=c(6);            % risk premium for the first state process
l2=c(7);            % rist premium for the second state process
rho=sin(c(8));      % correlation between pocesses
p=[abs(c(9))+0.0001 0; 0 abs(c(10))+0.0001];    % covariance between states
%% Seasonality parameters
ph=c(11);               %phi
zeta=c(12);             %zeta
psi=c(13);              %psi
b=c(14);                %b  
expkd=exp(-k*delta);
%% Seasonality function
alpha = @(b,zeta,psi,ph,t) b+zeta*sin(psi*(t+delta)+ph);
%Initialise vectors and matrices for Kalman filter%
x_old=[0 zeros(1,m);x0 zeros(1,m)];
x_new=[zeros(1,m+1);zeros(1,m+1)];
%For more information see the paper%
B=[expkd 0; 0 1];
RR=[sigma1^2*0.5/k*(1-expkd^2) rho*sigma1*sigma2/k*(1-expkd);...
      rho*sigma1*sigma2/k*(1-expkd) sigma2^2*delta];
g=[0;a2*delta];
Af=@(t) [exp(-k*t) ones(length(t),1)];
%% For reducing the total number of calculation some values are calculated outside the loop
Tv=repmat(T',1,m+2)-repmat(0:m+1,n,1)*delta; % matrix with maturity times
dg=alpha(b,zeta,psi,ph,D)';                 % vector with seasonality values
d=l1*sigma1/k*(1-exp(-k*Tv))+(a2+l2*sigma2)*Tv+...
    sigma1^2*0.25/k*(1-exp(-2*k*Tv))+...
    rho*sigma1*sigma2/k*(1-exp(-k*Tv))+0.5*sigma2^2*Tv;

y=y';



switch lower(tag)
    case {'calibration'}
        z=0;
        for i=2:m-1
            A=Af(Tv(:,i+1));                    % Update    matrix A
            x_new(:,i-1)=B*x_old(:,i-1)+g;      % stae estimate
            v=y(:,i)-(A*(x_new(:,i-1))+d(:,i)+dg);  % Innovation
            sig=A*p*A'+QQ;                      % Innovation covariance
            k=p*A'/sig;                         % Kalman gain
            x_old(:,i)=x_new(:,i-1)+k*v;        % state estimate posteriri update
            p=B*p*B'+RR-B*p*A'*(sig\A)*p*B';    % covarince mmatric posteriori update
            z=z+log(det(sig))+v'*(sig\v);       % summ up MLE
        end;
       
    case {'test'}
            commodity=B*x_old(:,1)+g;   %state estimate apriori
            z=Af(Tv(:,2))*commodity+d(:,2)+dg;  % sutures state estimate apriori
            commodity=sum(commodity);      % sum of the states as commodity estimate 
end;        
        
        