%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Author Suren Islyaev
%Function commopdityforecast3
%Input:
%     c - vector of parameters
%     x - vector of commodity prices (daily)
%     delta - timestep annulised
%      tag - type of output if tag='calibration' - outputs MLE, otherwise
%      outputs commodity price
% Output:
%     Theta - vector of parameters
%     RAE - Vector of daily relative errors
%     MRAE - Mean relative error for one step ahead forecast
%     fval - value of log MLE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function z=commodityforecast3(c,x,delta,tag)
%Initialise function parameters
m0=c(1);
k=abs(c(2));
sigma=c(3)^2;
theta=c(4)^2;
St=x(2:end); % vector of spot prices
if length(x)>1  % if x is not scalar it takes following value
    E=x(1:end-1)*exp(-k*delta)+m0/k*(1-exp(-k*delta));
else %otherwise:
     E=x*exp(-k*delta)+m0/k*(1-exp(-k*delta));
end;
%Variance:
Var=theta/k/2*(1-exp(-k*delta))^2 +sigma^2/k*.5*(1-exp(-2*k*delta));
if strcmp(tag,'calibration') % if tag is calibration calculate MLE:
    fval=((St-E).^2)./Var+log(Var);
    z=sum(fval);
else% get the Expected value of x
    z=E;
end;
