%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Author Suren Islyaev
%Function main
%Input:
%     myfunction - pointer to function desribing the behaviour of commodity 
%                   prices
%     data    - Data 
%     x0  - Initial guess 
%     Precision - Precision 
%     samplesize - In-sample size
%     testsample - OutOfSanmple size
% Output:
%     Theta - vector of parameters
%     RAE - Vector of daily relative errors
%     MRAE - Mean relative error for one step ahead forecast
%     fval - value of log MLE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    


function [Theta,RAE,MRAE,RMSE,fval] = main(myfunction,data, x0, Precision, samplesize,testsample )
%[n,m]=size(data);
n_new=samplesize+testsample;
%% Initial values
delta=1/252; %timestep weekly
x=log(data(1:n_new,1));    %vector of commodity prices price at time t=0
y=x;
n=length(x)-1;

%% Optimisation properties settings

    options=optimset('MaxFunEvals',	Precision,...
                     'MaxIter',Precision,...
                     'UseParallel','always',...
                     'TolFun',1e-16);

                     
    [Theta, fval]=fminsearch(@(c) myfunction(c,y(1:...
    samplesize,:),delta,'calibration'),x0,options);

%% Initialise output
commodity=zeros(n-samplesize-1,1);
k=0;
%% Generating the forecast
for i=samplesize:n-1
[commodity(k+1)]=myfunction(Theta,x(samplesize+k),delta,'test');
k=k+1;
end;
 %%
 realComm=exp(x(samplesize+1:n));
 comcum=exp(commodity(1:end));
 X=realComm-comcum;
 N=length(X);
 RAE=abs(X)./realComm*100;
 MRAE=mean(RAE);
 RMSE=sqrt(sum((X).^2)/N);
end

