function CallValue = hestoncall(kappa,theta,sigma,rho,v0,r,T,s0,strike) 
% This is for in the money and at the money
%% Parameters
% kappa = rate of reversion
% theta = long run variance
% sigma = volatility of volatility
% v0 = initial Variance
% rho = correlation
% T = Time till maturity
% r = interest rate
% s0 = initial asset price
x0 = log(s0);
alpha = 1.25;
N= 4096*2;
% N= 4096;
c = 800;           % upper limit of integration
eta = c/N;
b =pi/eta;
L=1:N;
u = (L-1)*eta;   % # of points to integrate in CF
lambda = 2*b/N;
position = (log(strike) + b)/lambda + 1; %position of call value in FFT
v = u - (alpha+1)*1i;

%% Hong (2004)'s SV CF
zeta = -.5*(v.^2 +1i*v);
gamma = kappa - rho*sigma*v*1i;
PHI = sqrt(gamma.^2 - 2*sigma^2*zeta);
A = 1i*v*(x0 + r*T);
B = v0*((2*zeta.*(1-exp(-PHI.*T)))./(2*PHI - ...
(PHI-gamma).*(1-exp(-PHI*T))));
C = -kappa*theta/sigma^2*(2*log((2*PHI - ...
(PHI-gamma).*(1-exp(-PHI*T)))./ ...
(2*PHI)) + (PHI-gamma)*T);

%% Modified CF with FFT algorithm
charFunc = exp(A + B + C);
ModifiedCharFunc = charFunc*exp(-r*T)./(alpha^2 ...
+ alpha - u.^2 + 1i*(2*alpha +1)*u);
SimpsonW = 1/3*(3 + (-1i).^L - [1, zeros(1,N-1)]);
FftFunc = exp(1i*b*u).*ModifiedCharFunc*eta.*SimpsonW;
payoff = real(fft(FftFunc));
CallValueM = exp(-log(strike)*alpha)*payoff/pi;
format short;
CallValue = CallValueM(round(position));