function batesfft = BatesFFT(KAPPA,THETA,SIGMAv,RHO,V0,LAMBDA,MUJ,SIGMAS,r,q,T,S0,K)
alpha = 0.75;
N = 2^12;
a = 600; %Upper limit of integration
eta = a/N; %Grid spacing for integration
lambda = (2*pi) / (N*eta); %Width of intervals between successive strikes
b = N*lambda / 2;
if S0 >= K %For ITM and ATM options
u = (0:(N-1)) * eta; %Integration grid
v = u - (alpha + 1) * 1i;
%Characteristic Function Variables:
ALPHA = -0.5*(v.^2 + 1i*v);
BETA = KAPPA - RHO*SIGMAv*1i*v;
GAMMA = (SIGMAv^2)/2;
d = sqrt(BETA.^2 - 4*ALPHA*GAMMA);
rpos = (BETA + d)/(SIGMAv^2);
rneg = (BETA - d)/(SIGMAv^2);
g = rneg./rpos;
D = rneg .* ((1 - exp(-d*T)) ./ (1 - g.*exp(-d*T)));
C = KAPPA * (rneg*T - (2/(SIGMAv^2)) * log((1 - g.*exp(-d*T))...
./ (1 - g)));
%Characteristic Function and Fourier Transform
CharFun = exp(C*THETA + D*V0 + 1i*v*(log(S0) + (r-q)*T));
%Diffusion characteristic function
JumpCFn = exp(-LAMBDA*MUJ*1i*v*T + LAMBDA*T * ((1 + MUJ).^(1i.*v)...
.* exp(SIGMAS^2*(1i.*v/2).*(1i.*v - 1)) - 1));

%Jump characteristic function
FourierTrans = (exp(-r*T) * CharFun .* JumpCFn) ./ ((alpha + 1i*u)...
.* (alpha + 1i*u + 1));
SWeightings = (1/3) * (3 + (-1).^(1:N) - [1 zeros(1,N-1)]);
%Include Simpson's weightings
FFT = exp(1i*b*u) .* FourierTrans * eta .* SWeightings;
%FFT Routine
FFT = real(fft(FFT)); %Call MATLAB FFT routine
%Call Price Calculation
strikes = -b + lambda*(0:N-1); %Log-strike price grid
batesfft = (exp(-strikes*alpha)/pi) .* FFT; %Include dampening factor
position = (log(K) + b) / lambda + 1; %Strike position on grid
batesfft = (1-(position-floor(position))) * ...
batesfft(floor(position)) + (position-floor(position)) ...
* batesfft(floor(position)+1);
%Interpolated FFT call price
elseif S0 < K %For OTM options
u = (0:(N-1)) * eta; %Integration grid
v1 = u - 1i*alpha;
v2 = u + 1i*alpha;
w1 = u - 1i*alpha - 1i;
w2 = u + 1i*alpha - 1i;
%Characteristic Function 1 Variables
ALPHA1 = -0.5*(w1.^2 + 1i*w1);
BETA1 = KAPPA - RHO*SIGMAv*1i*w1;
GAMMA1 = (SIGMAv^2)/2;
d1 = sqrt(BETA1.^2 - 4*ALPHA1*GAMMA1);
rpos1 = (BETA1 + d1)/(SIGMAv^2);
rneg1 = (BETA1 - d1)/(SIGMAv^2);
g1 = rneg1./rpos1;
D1 = rneg1 .* ((1 - exp(-d1*T)) ./ ...
(1 - g1.*exp(-d1*T)));

C1 = KAPPA * (rneg1*T - (2/(SIGMAv^2)) * ...
log((1 - g1.*exp(-d1*T)) ./ (1 - g1)));
JumpCFn1 = exp(-LAMBDA*MUJ*1i*w1*T + LAMBDA*T * ((1 + MUJ).^(1i*w1)...
.* exp(SIGMAS^2*(1i.*w1/2).*(1i.*w1 - 1)) - 1));
%Jump characteristic function
CharFun1 = exp(C1*THETA + D1*V0 + 1i*w1*(log(S0) + (r-q)*T)) ...
.* JumpCFn1;
%Characteristic Function 2 Variables
ALPHA2 = -0.5*(w2.^2 + 1i*w2);
BETA2 = KAPPA - RHO*SIGMAv*1i*w2;
GAMMA2 = (SIGMAv^2)/2;
d2 = sqrt(BETA2.^2 - 4*ALPHA2*GAMMA2);
rpos2 = (BETA2 + d2)/(SIGMAv^2);
rneg2 = (BETA2 - d2)/(SIGMAv^2);
g2 = rneg2./rpos2;
D2 = rneg2 .* ((1 - exp(-d2*T)) ./ ...
(1 - g2.*exp(-d2*T)));
C2 = KAPPA * (rneg2*T - (2/(SIGMAv^2)) * ...
log((1 - g2.*exp(-d2*T)) ./ (1 - g2)));
JumpCFn2 = exp(-LAMBDA*MUJ*1i*w2*T + LAMBDA*T * ((1 + MUJ).^(1i*w2)...
.* exp(SIGMAS^2*(1i.*w2/2).*(1i.*w2 - 1)) - 1));
%Jump characteristic function
CharFun2 = exp(C2*THETA + D2*V0 + 1i*w2*(log(S0) + (r-q)*T)) ...
.* JumpCFn2;
%Characteristic Function and Fourier Transform
zeta1 = exp(-r*T) * ((1./(1 + 1i*v1)) - exp(r*T)./(1i*v1) ...
- CharFun1./(v1.^2 - 1i*v1));
zeta2 = exp(-r*T) * ((1./(1 + 1i*v2)) - exp(r*T)./(1i*v2) ...
- CharFun2./(v2.^2 - 1i*v2));
FourierTrans = (zeta1 - zeta2) / 2;
SWeightings = (1/3) * (3 + (-1).^(1:N) - [1 zeros(1,N-1)]);
%Include Simpson's weightings
FFT = exp(1i*b*u) .* FourierTrans * eta .* SWeightings;

%FFT Routine
FFT = real(fft(FFT)); %Call MATLAB FFT routine
%Call Price Calculation
strikes = -b + lambda*(0:N-1); %Log-strike price grid
batesfft = (1 ./ (pi*sinh(alpha*strikes))) .* FFT;
%Include dampening factor
position = (log(K) + b) / lambda + 1; %Strike position on grid
batesfft = (1-(position-floor(position))) * ...
batesfft(floor(position)) + (position-floor(position)) ...
* batesfft(floor(position)+1);
%Interpolated FFT call price
end
