function [Price,conf, Time] = hestonmontecarlo(KAPPA,THETA,SIGMAv,RHO,V0,r,T,S0,K,PATHS,n,barrier)
Timer = clock; %Start the algorithm timer
dt = T/n; %Interval length
V = [V0 zeros(1,n)]; %Initialise variance vector
S = [S0 zeros(1,n)]; %Initialise price vector
%Simulate the diffusion part of the stock price process:
payoff=zeros(PATHS,1);
for j=1:PATHS
    Zv = randn(1,n+1);
    Zs = RHO*Zv+sqrt(1-RHO^2).*randn(1,n+1);
    for i = 2:n+1;
        S(i) = S(i-1)+ S(i-1)*r*dt + S(i-1).*sqrt(max(V(i-1),0)).*Zs(i-1)*sqrt(dt);
        V(i) = V(i-1)+KAPPA.*(THETA-V(i-1)).*dt+SIGMAv.*sqrt(max(V(i-1),0)).*Zv(i-1)*sqrt(dt);
        if S(i)<=barrier
            break
        end;
    end
    if i==(n+1)
        payoff(j)=max(S(end)-K,0);
    end;
end;


Price = mean(payoff)*exp(-r*T); %Call option price
bm=std(payoff);
conf=[Price-1.96*bm/sqrt(PATHS),Price+1.96*bm/sqrt(PATHS)];
Time = etime(clock,Timer); %Stop the algorithm time