function [Price,conf, Time] = batesmontecarlo_asian(k,theta,SIGMAv,RHO,V0,MUJ,...
SIGMAS,LAMBDA,r,T,S0,PATHS,n)
Timer = clock; %Start the algorithm timer
dt = T/n; %Interval length
%A = zeros(1,n+1); %Initialise variance vector
%S = [S0 zeros(1,n)]; %Initialise price vector

payoff=zeros(PATHS,1);
parfor j=1:PATHS
    S=S0; V=V0; Sm=S0;
%Simulate the diffusion part of the stock price process:
    Zv = randn(1,n+1);
    Zs = RHO*Zv+sqrt(1-RHO^2).*randn(1,n+1);
%Simulate the jump part of the stock price process:
    PoissonRVs = poissrnd(LAMBDA*dt,1,n+1); %Randomly generate the
%number of jumps occurring along each simulated stock path
    NT = lognrnd(log(1+MUJ)-0.5*SIGMAS^2,SIGMAS,1,n+1);
    C=(NT-1).*PoissonRVs;
    for i = 2:n+1;
        S = S+S*C(i-1) + S*(r - LAMBDA*MUJ)*dt + S.*sqrt(max(V,0)).*Zs(i-1)*sqrt(dt);
        V = V+k.*(theta-V).*dt+SIGMAv.*sqrt(max(V,0)).*Zv(i-1)*sqrt(dt);
        Sm=Sm+S;
    end;
%        A(j)=Sm/n;
        payoff(j)=max(S-Sm/n,0);
end;




Price = mean(payoff)*exp(-r*T); %Call option price
bm=std(payoff);
conf=[Price-1.96*bm/sqrt(PATHS),Price+1.96*bm/sqrt(PATHS)];
Time = etime(clock,Timer); %Stop the algorithm time